package org.eclipse.jst.jsf.validation.el.tests.jsp;

import java.util.List;

import org.eclipse.jdt.core.Signature;
import org.eclipse.jst.jsf.core.IJSFCoreConstants;
import org.eclipse.jst.jsf.validation.el.tests.base.SingleJSPTestCase;
import org.eclipse.jst.jsf.validation.internal.el.diagnostics.DiagnosticFactory;

/**
 * Test cases for logical not equals (ne, !=)
 * 
 * @author cbateman
 */
public class LogicalNotEqualsTestCase extends SingleJSPTestCase 
{
    public LogicalNotEqualsTestCase()
    {
        super("/testdata/jsps/logicalNotEquals.jsp.data", "/logicalNotEquals.jsp", IJSFCoreConstants.FACET_VERSION_1_1,FACES_CONFIG_FILE_NAME_1_1);
    }

    protected void setUp() throws Exception
    {
        super.setUp();
    }

    public void testSanity()
    {
        assertEquals("myBean.stringProperty != '3'", getELText(_structuredDocument,852));
        assertEquals("myBean.stringProperty ne '3'", getELText(_structuredDocument,914));
        assertEquals("myBean.integerProperty != 3", getELText(_structuredDocument,976));
        assertEquals("myBean.integerProperty ne 3", getELText(_structuredDocument,1037));
        assertEquals("myBean.booleanProperty != true", getELText(_structuredDocument,1098));
        assertEquals("myBean.booleanProperty ne true", getELText(_structuredDocument,1162));
        assertEquals("myBean.integerProperty != '4' ", getELText(_structuredDocument,1226));
        assertEquals("myBean.integerProperty ne '4' ", getELText(_structuredDocument,1290));
        assertEquals("myBean.bigDoubleProperty != 4.5", getELText(_structuredDocument,1354));
        assertEquals("myBean.doubleProperty ne 67", getELText(_structuredDocument,1419));
        assertEquals("myBean.bigIntegerProperty != 500", getELText(_structuredDocument,1480));
        assertEquals("myBean.stringArrayProperty != '3'", getELText(_structuredDocument,1546));
        assertEquals("myBean.stringArrayProperty ne myBean.listProperty", getELText(_structuredDocument,1613));
        assertEquals("myBean.coins != 'dime'", getELText(_structuredDocument,1692));
        assertEquals("myBean.coins ne 'dime'", getELText(_structuredDocument,1745));
        assertEquals("myBean.colors != 'red'", getELText(_structuredDocument,1798));
        assertEquals("myBean.colors ne 'red'", getELText(_structuredDocument,1850));
        assertEquals("myBean.coins != myBean.stringProperty", getELText(_structuredDocument,1902));
        assertEquals("myBean.coins ne myBean.stringProperty", getELText(_structuredDocument,1969));
        assertEquals("myBean.rawEnum != 'red'", getELText(_structuredDocument,2036));
        assertEquals("myBean.coinEnum ne myBean.coins", getELText(_structuredDocument,2089));

        assertEquals("5 != 3", getELText(_structuredDocument,2175));
        assertEquals("5 ne 3", getELText(_structuredDocument,2212));
        assertEquals("true != false", getELText(_structuredDocument,2249));
        assertEquals("true ne false", getELText(_structuredDocument,2293));
        assertEquals("'4' != '34'", getELText(_structuredDocument,2337));
        assertEquals("'4' ne '34'", getELText(_structuredDocument,2379));
        assertEquals("'34' != '34'", getELText(_structuredDocument,2421));
        assertEquals("'34' ne '34'", getELText(_structuredDocument,2464));
        assertEquals("myBean.integerProperty != null", getELText(_structuredDocument,2507));
        assertEquals("null ne myBean.integerProperty", getELText(_structuredDocument,2568));
        assertEquals("5.4 != 4.3", getELText(_structuredDocument,2629));
        assertEquals("true != true", getELText(_structuredDocument,2670));
        assertEquals("myBean.coins != 'notAValue'", getELText(_structuredDocument,2713));
        assertEquals("myBean.coins ne 'notAValue'", getELText(_structuredDocument,2771));
        assertEquals("myBean.coins != 'notAValue' || myBean.coins != 'dime'", getELText(_structuredDocument,2829));
        assertEquals("myBean.coins ne 'notAValue' || myBean.coins ne 'dime'", getELText(_structuredDocument,2913));
        assertEquals("myBean.coins != myBean.colors", getELText(_structuredDocument,2997));

        assertEquals("5 != true", getELText(_structuredDocument,3078));
        assertEquals("5 ne true", getELText(_structuredDocument,3118));
        assertEquals("myBean.integerProperty != myBean.booleanProperty", getELText(_structuredDocument,3158));
        assertEquals("myBean.integerProperty ne myBean.booleanProperty", getELText(_structuredDocument,3237));
        assertEquals("myBean.stringArrayProperty != myBean.booleanProperty", getELText(_structuredDocument,3316));
        assertEquals("myBean.booleanProperty ne myBean.stringArrayProperty", getELText(_structuredDocument,3399));
        assertEquals("myBean.integerProperty != true ", getELText(_structuredDocument,3485));
        assertEquals("myBean.integerProperty ne true ", getELText(_structuredDocument,3550));
        assertEquals("false != myBean.integerProperty", getELText(_structuredDocument,3615));
    }

    public void testNoErrorExprs() 
    {
        assertNoError(852, Signature.SIG_BOOLEAN);
        assertNoError(914, Signature.SIG_BOOLEAN);
        assertNoError(976, Signature.SIG_BOOLEAN);
        assertNoError(1037, Signature.SIG_BOOLEAN);
        assertNoError(1098, Signature.SIG_BOOLEAN);
        assertNoError(1162, Signature.SIG_BOOLEAN);
        assertNoError(1226, Signature.SIG_BOOLEAN);
        assertNoError(1290, Signature.SIG_BOOLEAN);
        assertNoError(1354, Signature.SIG_BOOLEAN);
        assertNoError(1419, Signature.SIG_BOOLEAN);
        assertNoError(1480, Signature.SIG_BOOLEAN);
        assertNoError(1546, Signature.SIG_BOOLEAN);
        assertNoError(1613, Signature.SIG_BOOLEAN);
        assertNoError(1692, Signature.SIG_BOOLEAN);
        assertNoError(1745, Signature.SIG_BOOLEAN);
        assertNoError(1798, Signature.SIG_BOOLEAN);
        assertNoError(1850, Signature.SIG_BOOLEAN);
        assertNoError(1902, Signature.SIG_BOOLEAN);
        assertNoError(1969, Signature.SIG_BOOLEAN);
        assertNoError(2036, Signature.SIG_BOOLEAN);
        assertNoError(2089, Signature.SIG_BOOLEAN);
    }

    public void testWarningExprs() 
    {
        List list = assertSemanticWarning(2175, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);
        
        list = assertSemanticWarning(2212, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2249, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2293, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2337, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2379, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2421, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2464, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2507, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_EQUALITY_COMP_WITH_NULL_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2568, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_EQUALITY_COMP_WITH_NULL_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2629, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2670, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CONSTANT_EXPRESSION_ALWAYS_EVAL_SAME_ID);

        list = assertSemanticWarning(2713, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_COMPARISON_WITH_ENUM_ALWAYS_SAME_ID);

        list = assertSemanticWarning(2771, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_COMPARISON_WITH_ENUM_ALWAYS_SAME_ID);

        list = assertSemanticWarning(2829, Signature.SIG_BOOLEAN, 2);
        assertContainsProblem(list, DiagnosticFactory.BINARY_COMPARISON_WITH_ENUM_ALWAYS_SAME_ID);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_FIRST_ARGUMENT_SHORT_CIRCUITS_ID);

        list = assertSemanticWarning(2913, Signature.SIG_BOOLEAN, 2);
        assertContainsProblem(list, DiagnosticFactory.BINARY_COMPARISON_WITH_ENUM_ALWAYS_SAME_ID);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_FIRST_ARGUMENT_SHORT_CIRCUITS_ID);

        list = assertSemanticWarning(2997, Signature.SIG_BOOLEAN, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_COMPARISON_WITH_ENUM_ALWAYS_SAME_ID);
    }

    public void testErrorExprs() 
    {
        List list = assertSemanticError(3078, null, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_COULD_NOT_MAKE_NUMERIC_COERCION_ID);

        list = assertSemanticError(3118, null, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_COULD_NOT_MAKE_NUMERIC_COERCION_ID);

        list = assertSemanticError(3158, null, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_COULD_NOT_MAKE_NUMERIC_COERCION_ID);

        list = assertSemanticError(3237, null, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_COULD_NOT_MAKE_NUMERIC_COERCION_ID);

        list = assertSemanticError(3316, null, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CANNOT_COERCE_ARGUMENT_TO_BOOLEAN_ID);

        list = assertSemanticError(3399, null, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_CANNOT_COERCE_ARGUMENT_TO_BOOLEAN_ID);

        list = assertSemanticError(3485, null, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_COULD_NOT_MAKE_NUMERIC_COERCION_ID);

        list = assertSemanticError(3550, null, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_COULD_NOT_MAKE_NUMERIC_COERCION_ID);
        
        list = assertSemanticError(3615, null, 1);
        assertContainsProblem(list, DiagnosticFactory.BINARY_OP_COULD_NOT_MAKE_NUMERIC_COERCION_ID);
    }
}
